#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include <unistd.h>
#include <getopt.h>
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"
#include "pvo-metadata.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using namespace PVO::Meta;
using std::string;


int main(int argc, char * argv[])
{
  /** 
   * PVOSummary is a function to print out basic information for
   * PSRFITS files.
   *
   * This program is a command-line program that looks at PSRFITS
   * files (i.e. files that have extensions .fits or .rf) and returns
   * a summary of their positions, beam sizes and observation times.
   *
   * The format of usage is: <br> 
   * <tt>> pvosummary </tt><br>
   * to analyse all PSRFITS files in the current directory, or <br>
   * <tt>> pvosummary FILE</tt><br>
   * to analyse the file FILE. Entering<br>
   * <tt>> pvosummary -h</tt><br>
   * will give a help message, using PVO::pvosummary_usage_text.
   *
   * The output is a list of valid files, their central RA & DEC,
   * their beam size, the central observing frequency, and the
   * DATE-OBS keyword (time/date of observation),
   * 
   */


  bool verboseflag = false;
  bool makeCSV = false;
  bool quietMode = false;
  std::string csvFile;

  std::vector<std::string> namelist;
  unsigned int i;

  int c, digit_optind = 0;
  while (true) {
    int option_index = 0;
    static struct option long_options[] = {
      {"help", 0, 0, 'h'},
      {"csv", 1, 0, 'c'},
      {0, 0, 0, 0}
    };

    c = getopt_long (argc, argv, "c:hpq",
		     long_options, &option_index);

    if (c == -1) break;
    
    switch (c) {               
    case 0:
      // This is where we deal with -- options that don't have a
      // single letter equivalent
      break;
    case 'c': // make a csv file.
      makeCSV = true;
      csvFile = optarg;
      break;
    case 'h': // give help statement
      std::cout << pvosummary_usage_text;
      return 0;
      break;
    case 'p': // list the parameters
      std::cout << Meta::metadataList;
      return 0;
      break;
    case 'q': // quiet mode -- nothing written to screen
      quietMode = true;
      break;
    case '?': // invalid option
    case ':': // missing an argument
      std::cout << pvosummary_usage_text_short;
      return 1;
      break;
    default:
      std::cerr << "ERROR: getopt returned character code " <<  c << "\n";
      break;
    }

  }

  if (optind < argc) { 
    // There is a non-option argument -- must be the file to access
    while (optind < argc) namelist.push_back( argv[optind++] );
  }
  else{ 
    //No file has been specified, so we are working on the directory
    //as a whole.
    glob_t globbuf1,globbuf2;
    globbuf1.gl_offs = 0;
    globbuf2.gl_offs = 0;
    glob("*.fits", GLOB_DOOFFS, NULL, &globbuf1);
    glob("*.rf", GLOB_DOOFFS, NULL, &globbuf2);
    namelist.resize(globbuf1.gl_pathc+globbuf2.gl_pathc);
    for(i=0;i<globbuf1.gl_pathc+globbuf1.gl_offs;i++) 
      namelist[i] = globbuf1.gl_pathv[i];
    for(i=0;i<globbuf2.gl_pathc+globbuf2.gl_offs;i++) 
      namelist[globbuf1.gl_pathc+i] = globbuf2.gl_pathv[i];
  }

  std::vector<Metadata> goodlist;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i]);
  
    if(status==0){
      Metadata newmatch;
      newmatch.define(fitsfile);
      goodlist.push_back(newmatch);
    }
  }

  if(!quietMode) printMetadataShort(std::cout,goodlist);

  if(makeCSV) printMetadataFull(csvFile, goodlist);

}
